package com.persagy.person.controller;

import java.util.List;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Lists;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.utils.ResponseResult;
import com.persagy.common.utils.ResponseResultUtil;
import com.persagy.common.utils.StringUtil;
import com.persagy.person.manage.SaasRoleMenuHandler;
import com.persagy.person.pojo.dto.SaasFunction;
import com.persagy.person.pojo.dto.SaasMenu;
import com.persagy.person.pojo.dto.SaasRole;
import com.persagy.person.pojo.vo.function.SaasFunctionCreateVO;
import com.persagy.person.pojo.vo.function.SaasFunctionDeleteVO;
import com.persagy.person.pojo.vo.function.SaasFunctionQueryVO;
import com.persagy.person.service.ISaasFunctionService;
import com.persagy.person.service.ISaasMenuService;
import com.persagy.person.service.ISaasRoleService;

import cn.hutool.core.collection.CollectionUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 功能信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */

@Api(tags = "功能信息")
@RestController
@RequestMapping(value = "/function", method = RequestMethod.POST)
public class SaasFunctionController {
	
    @Autowired
    private ISaasMenuService saasMenuService;
    
    @Autowired
    private ISaasRoleService saasRoleService;
    
    @Autowired
    private ISaasFunctionService saasFunctionService;
    
    @Autowired
    private SaasRoleMenuHandler saasRoleMenuHandler;
    
    /**
     * 功能点列表查询
     */
    @ApiOperation(value = "功能点列表查询")
    @RequestMapping(value = "querySaasFunctionList")
    public ResponseResult querySaasFunctionList(@RequestBody @Validated SaasFunctionQueryVO queryVO) {
    	LambdaQueryWrapper<SaasFunction> queryWrapper = new SaasFunction.Builder().createQueryWrapper()
    			.menuIdEq(queryVO.getMenuId()).funCodeEq(queryVO.getFunCode()).updateUserEq(queryVO.getUpdateUser())
    			.funNameLike(queryVO.getFunName()).builderQueryWrapper();
    	queryWrapper.orderByAsc(SaasFunction::getFunSort);
    	List<SaasFunction> list = this.saasFunctionService.list(queryWrapper);
    	if (CollectionUtil.isEmpty(list)) {
			return ResponseResultUtil.successResult(Lists.newArrayList(), 0L);
		}
        return ResponseResultUtil.successResult(list, (long)list.size());
    }
    
    /**
     * 角色功能点列表查询，根据角色做权限过滤
     */
    @ApiOperation(value = "角色菜单功能点列表查询")
    @RequestMapping(value = "querySaasRoleFunctions")
    public ResponseResult querySaasRoleFunctions(@RequestBody @Validated SaasFunctionQueryVO queryVO) {
    	if (StringUtil.isBlank(queryVO.getRoleId())) {
    		return ResponseResultUtil.errorResult(ResponseCode.A0402.getCode(), "角色ID，不可为空");
		}
    	// 验证角色是否存在
    	SaasRole saasRole = this.saasRoleService.getOne(queryVO.getRoleId());
    	if (saasRole == null) {
    		return ResponseResultUtil.errorResult(ResponseCode.A0402.getCode(), "角色信息不存在");
		}
    	// 优先查询出存在功能点的信息集合
    	List<SaasFunction> functions = this.saasFunctionService.queryRoleFunctionList(queryVO.getGroupCode(), queryVO.getRoleId(), null);
    	// 再查询出此角色下，所有不存在功能点的菜单集合
    	List<SaasMenu> menus = this.saasMenuService.queryMenuAndEmptyFunction(queryVO.getGroupCode(), queryVO.getRoleId());
    	
    	JSONObject result = new JSONObject();
    	result.put("menus", menus == null ? Lists.newArrayList() : menus);
    	result.put("functions", functions == null ? Lists.newArrayList() : functions);
        return ResponseResultUtil.successResult(result);
    }

    /**
     * 新增
     */
	@ApiOperation(value = "保存")
	@RequestMapping(value = "createSaasFunction")
    public ResponseResult createSaasFunction(@RequestBody @Validated SaasFunctionCreateVO createVO) {
		// 验证菜单ID是否存在
		SaasMenu menu = this.saasMenuService.getById(createVO.getMenuId());
		if (menu == null) {
			return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "不存在此菜单");
		}
		SaasFunction saasFunction = new SaasFunction();
		BeanUtils.copyProperties(createVO, saasFunction);
		saasFunction.setId(null);
		saasFunction.setUpdateUser(createVO.getAccountId());
        boolean result = this.saasFunctionService.save(saasFunction);
        return result ? ResponseResultUtil.successResult("保存成功", saasFunction.getId()) : ResponseResultUtil.errorResult("保存失败");
    }
	
    /**
     * 更新
     */
	@ApiOperation(value = "更新")
	@RequestMapping(value = "updateSaasFunction")
    public ResponseResult updateSaasFunction(@RequestBody @Validated SaasFunctionCreateVO createVO) {
		if (StringUtil.isBlank(createVO.getId())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "功能ID不可为空");
		}
		// 验证菜单ID是否存在
		SaasMenu menu = this.saasMenuService.getById(createVO.getMenuId());
		if (menu == null) {
			return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "不存在此菜单");
		}
		SaasFunction saasFunction = new SaasFunction();
		BeanUtils.copyProperties(createVO, saasFunction);
		saasFunction.setUpdateUser(createVO.getAccountId());
        boolean result = this.saasFunctionService.updateById(saasFunction);
        return result ? ResponseResultUtil.successResult("更新成功") : ResponseResultUtil.errorResult("更新失败");
    }

    /**
     * 物理删除
     */
    @ApiOperation(value = "删除")
    @RequestMapping(value = "deleteSaasFunction")
    public ResponseResult deleteSaasFunction(@RequestBody @Validated SaasFunctionDeleteVO deleteVO) {
    	if (StringUtil.isAllBlank(deleteVO.getId(), deleteVO.getMenuId())) {
    		return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "功能ID与菜单ID至少存在其一");
		}
        boolean result = this.saasRoleMenuHandler.deleteSaasFunction(deleteVO.getId(), deleteVO.getMenuId());
        return result ? ResponseResultUtil.successResult("删除成功") : ResponseResultUtil.errorResult("删除失败");
    }
    
}
